@@ -21,7 +21,6 @@ except ImportError:
 
 from typing import Dict, List, Mapping, Optional, Tuple, Union
 
-from cache_utils.decorators import cached
 from datetime_truncate import truncate
 from dateutil.relativedelta import relativedelta
 from dateutil.rrule import DAILY, HOURLY, MONTHLY, WEEKLY, YEARLY, rrule
@@ -39,6 +38,7 @@ from django.dispatch import receiver
 from django.utils import timezone
 from django.utils.safestring import mark_safe
 from django.utils.translation import gettext_lazy as _
+from memoize import delete_memoized, memoize
 from multiselectfield import MultiSelectField
 
 
@@ -421,6 +421,7 @@ class DashboardStats(models.Model):
         db_table = "dashboard_stats"
         verbose_name = _("dashboard stats")
         verbose_name_plural = _("dashboard stats")
+        permissions = (("view_dashboard_stats", _("Can view dashboard charts")),)
 
     def allowed_type_operation_field_name_choices(self):
         return (op for op in operation if op[0] in self.allowed_type_operation_field_name)
@@ -879,15 +880,12 @@ class CriteriaToStatsM2M(models.Model):
         query = model.objects.all().query
         return query.resolve_ref(field_name).field
 
-    # The slef argument is here just because of this bug:
-    # https://github.com/infoscout/django-cache-utils/issues/19
-    @cached(60 * 60 * 24 * 7)
+    @memoize(60 * 60 * 24 * 7)
     def _get_dynamic_choices(
         self,
-        slef,
-        time_since,
-        time_until,
-        count_limit=None,
+        time_since: datetime.datetime,
+        time_until: datetime.datetime,
+        count_limit: int = None,
         operation_choice=None,
         operation_field_choice=None,
         user=None,
@@ -988,7 +986,6 @@ class CriteriaToStatsM2M(models.Model):
             time_since = None
             time_until = None
         choices = self._get_dynamic_choices(
-            self,
             time_since,
             time_until,
             self.count_limit,
@@ -1058,15 +1055,15 @@ class CachedValue(models.Model):
 @receiver(post_save, sender=DashboardStatsCriteria)
 def clear_caches_criteria(sender, instance, **kwargs):
     for m2m in instance.criteriatostatsm2m_set.all():
-        m2m._get_dynamic_choices.invalidate(m2m)
+        delete_memoized(m2m._get_dynamic_choices)
 
 
 @receiver(post_save, sender=DashboardStats)
 def clear_caches_stats(sender, instance, **kwargs):
     for m2m in instance.criteriatostatsm2m_set.all():
-        m2m._get_dynamic_choices.invalidate(m2m)
+        delete_memoized(m2m._get_dynamic_choices)
 
 
 @receiver(post_save, sender=CriteriaToStatsM2M)
 def clear_caches_stats_m2m(sender, instance, **kwargs):
-    instance._get_dynamic_choices.invalidate(instance)
+    delete_memoized(instance._get_dynamic_choices)
