@@ -17,6 +17,7 @@ from django.test.utils import override_settings
 from django.urls import reverse
 from model_bakery import baker
 
+from admin_tools_stats.models import DashboardStats
 from admin_tools_stats.views import AnalyticsView, ChartDataView, Interval
 
 from .utils import (
@@ -44,14 +45,35 @@ class AnalyticsViewTest(BaseSuperuserAuthenticatedClient):
             model_name="TestKid",
             model_app_name="demoproject",
             graph_key="kid_graph",
+            operation_field_name="height",
         )
         super().setUp()
 
+    def test_analytics_view_empty(self):
+        """Test of analytics page when no charts are present"""
+        DashboardStats.objects.all().delete()
+        response = self.client.get(reverse("chart-analytics"))
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(
+            response,
+            "<p>No charts available, please "
+            '<a href="/admin/admin_tools_stats/dashboardstats/">configure them</a></p>',
+            html=True,
+        )
+
     def test_analytics_view(self):
         """Test function to check dashboardstats admin pages"""
         response = self.client.get(reverse("chart-analytics"))
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, "<button>Kid chart</button>", html=True)
+        self.assertNotContains(response, "loadAnalyticsChart('")
+
+    def test_analytics_view_show(self):
+        """Test function to check dashboardstats admin pages that should show certain chart"""
+        response = self.client.get(reverse("chart-analytics") + "?show=kid_graph")
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, "<button>Kid chart</button>", html=True)
+        self.assertContains(response, "loadAnalyticsChart('kid_graph')")
 
     def test_analytics_chart_view(self):
         """Test function to check dashboardstats admin pages"""
@@ -152,6 +174,15 @@ class ChartDataViewContextTests(BaseSuperuserAuthenticatedClient):
             graph_key="user_graph",
             operation_field_name="is_active,is_staff",
         )
+        self.kid_stats = baker.make(
+            "DashboardStats",
+            graph_title="Kid chart",
+            date_field_name="birthday",
+            model_name="TestKid",
+            model_app_name="demoproject",
+            graph_key="kid_graph",
+            operation_field_name="height",
+        )
         self.request_factory = RequestFactory()
         super().setUp()
 
@@ -279,6 +310,63 @@ class ChartDataViewContextTests(BaseSuperuserAuthenticatedClient):
             },
         )
 
+    @override_settings(USE_TZ=True, TIME_ZONE="Europe/Prague")
+    def test_get_context_tz_operation(self):
+        """
+        Test function view rendering multi series with multiple operations
+        Test correct context in more complicated timezone setting
+        Set select_box_operation field
+        """
+        baker.make("TestKid", birthday=datetime(2021, 10, 30, tzinfo=timezone.utc), height=150)
+        baker.make("TestKid", birthday=datetime(2021, 10, 31, tzinfo=timezone.utc), height=160)
+        baker.make("TestKid", birthday=datetime(2021, 11, 1, tzinfo=timezone.utc), height=170)
+        baker.make("TestKid", birthday=datetime(2021, 11, 2, tzinfo=timezone.utc), height=180)
+        baker.make("TestKid", birthday=datetime(2021, 11, 3, tzinfo=timezone.utc), height=190)
+        baker.make("TestKid", birthday=datetime(2021, 11, 3, tzinfo=timezone.utc), height=210)
+        url = reverse("chart-data", kwargs={"graph_key": "kid_graph"})
+        url += (
+            "?time_since=2021-10-29&time_until=2021-11-05&select_box_interval=days&"
+            "select_box_chart_type=stackedAreaChart&select_box_operation_field=&debug=True&"
+            "select_box_operation=Avg"
+        )
+        chart_data_view = ChartDataView()
+        chart_data_view.request = self.request_factory.get(url)
+        chart_data_view.request.user = self.user
+        context = chart_data_view.get_context_data(graph_key="kid_graph")
+        self.assertDictEqual(
+            context,
+            {
+                "chart_container": "chart_container_kid_graph",
+                "chart_type": "stackedAreaChart",
+                "extra": {
+                    "tag_script_js": False,
+                    "use_interactive_guideline": True,
+                    "x_axis_format": "%d %b %Y",
+                    "x_is_date": True,
+                },
+                "values": {
+                    "extra1": {
+                        "date_format": "%a %d %b %Y",
+                        "tooltip": {"y_end": "", "y_start": ""},
+                    },
+                    "name0": "",
+                    "name1": Interval.days,
+                    "x": [
+                        1635458400000,  # 2021-10-28 22:00:00 GMT
+                        1635544800000,
+                        1635631200000,
+                        1635721200000,
+                        1635807600000,
+                        1635894000000,
+                        1635980400000,
+                        1636066800000,  # 2021-11-04 23:00:00 GMT
+                    ],
+                    "y0": [0, 150, 160, 170, 180, 200, 0, 0],
+                },
+                "view": chart_data_view,
+            },
+        )
+
 
 class SuperuserViewsTests(BaseSuperuserAuthenticatedClient):
     def setUp(self):
@@ -317,7 +405,7 @@ class SuperuserViewsTests(BaseSuperuserAuthenticatedClient):
         baker.make(
             "CachedValue",
             stats=self.stats,
-            time_scale=Interval.days,
+            time_scale="days",
             operation=None,
             dynamic_choices=[],
             filtered_value="",
